﻿using CaseAccountingContracts.BindingModels;
using CaseAccountingContracts.BusinessLogicContracts;
using CaseAccountingContracts.SearchModels;
using CaseAccountingContracts.StoragesContracts;
using CaseAccountingContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CaseAccountingBusinessLogic.BusinessLogics
{
    public class SpecializationLogic : ISpecializationLogic
    {
        private readonly ILogger _logger;
        private readonly ISpecializationStorage _specializationStorage;

        public SpecializationLogic(ILogger<SpecializationLogic> logger, ISpecializationStorage specializationStorage)
        {
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
            _specializationStorage = specializationStorage ?? throw new ArgumentNullException(nameof(specializationStorage));
        }

        public bool Create(SpecializationBindingModel model)
        {
            CheckModel(model);
            if (_specializationStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(SpecializationBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_specializationStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public SpecializationViewModel? ReadElement(SpecializationSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _specializationStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<SpecializationViewModel>? ReadList(SpecializationSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}", model?.Id);
            var list = model == null ? _specializationStorage.GetFullList() : _specializationStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool Update(SpecializationBindingModel model)
        {
            CheckModel(model);
            if (_specializationStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(SpecializationBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.UserId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор пользователя",
                    nameof(model.UserId));
            }
            if (model.Name == string.Empty) 
            {
                throw new ArgumentNullException("Некорректное название специализации",
                    nameof(model.Name));
            }
        }
    }
}
